"The comparations of number_comparator."

__all__ = ["PAIR_TERMINATIONS",
           "isPair",
           "isPeriodic",
           "isPrime",
           "isMultiple"]

PAIR_TERMINATIONS = ["2",
                     "4",
                     "6",
                     "8",
                     "0"]

from number_comparator.data.first_prime import first_168, PRIME_TERMINATIONS
import warnings

def isPair(n):
    "Look if an integer is a pair."
    if isinstance(n, type(1.0)):
        n = int(n)
    elif not isinstance(n, type(1)):
        raise ValueError(f"You must enter 'int' or 'float' numbers, not %s" % n)
    return str(n)[len(str(n)) - 1:] in PAIR_TERMINATIONS


def isPeriodic(a):
    "Check if a float/int is periodic."
    a = str(float(a)).split(".")[1]
    b = a[0]
    for i in a:
        r = i == b
    return not r


def isPrime(n):
    "Verify if a number is a prime by using the Eratostenes method."
    if not isinstance(n, (type(1), type(1.0))):
        raise ValueError(f"You must enter 'int' or 'float' numbers, not %s" % n)
    if n > first_168[len(first_168) - 1]:
        a = str(float(n))
        b = a[len(a) - 1] in PRIME_TERMINATIONS and n > 2
        return b is True and n not in [1001, 1003, 1007, 10001, 10003, 100001, 1000001] # These are not prime numbers
    return n in first_168


def truncate(n):
    "Truncate a number by converting it to an integer."
    if not isinstance(n, (type(1.0))):
        raise ValueError(f"You must enter 'float' numbers, not %s" % n)
    return int(n)


def roundout(n):
    "Roundout is a little bit more complex than truncate(), because we are depending from decimal output to operate."
    if not isinstance(n, (type(1.0))):
        raise ValueError(f"You must enter 'float' numbers, not %s" % n)


def isMultiple(a, b):
    "Verify if A is a multiple of B."
    if not isinstance(a, type(1)) or not isinstance(b, type(1)):
        raise TypeError(f"Expected int numbers, got values: %s | %s" % (a, b))
    elif b > a:
        raise SyntaxError(f"Value 1 (%s) must not be larger than value 2 (%s)" % (a, b))
    elif b < 1:
        raise ValueError(f"Cannot operate zero or negative values")
    elif b == 1:
        msg = f"""When using 1 as divisor (arg 2), you will always get True.
We are returning True immediatly to save variables."""
        warnings.warn(msg)
        return True
    c = 0
    d = False
    while c < a:
        c = c + b
        if c == a:
            d = True
    return d



"The operations of number_comparator."

__all__ = ["reverseNumber",
           "reverseFloat",
           "average",
           "averageFromList"]


"At first, try to import Colorama for Warnings."
from number_comparator.data.colors import Fore


def reverseNumber(n):
    "Get the reverse number for integers."
    if isinstance(n, type(1.0)):
        print(Fore.YELLOW + "WARNING: The 'reverseNumber' is only for integers. "
                       "The float number will be\n" + "truncated to integer. "
                       "If you are not agree, use function 'reverseFloat'.")
        n = int(n)
    try:
        if n < 0:
            return n + (n + n)
        elif n > 0:
            return n - (n + n)
        elif n == 0:
            raise ValueError(f"Cannot operate zero values")
        #else:
            #return None
    except (TypeError, ValueError):
        raise TypeError(f"'reverseNumber' function is only available for int or float numbers, not %s" % n)


def reverseFloat(n):
    "Get the reverse number for float numbers."
    if not isinstance(n, type(1.0)):
        raise TypeError(f"'Expected float numbers, not %s" % n)
    if n > 0.0:
        # Positive to negative
        return float("-" + str(n))
    elif n < 0.0:
        # Negative to positive
        return float(str(n)[1:])
    else:
        raise ValueError(f"Cannot operate zero values")


def average(*nums):
    "Get the average from a tuple."
    l = []
    for num in nums:
        if not isinstance(num, (type(1), type(1.0))):
            raise TypeError(f"Expected int/float numbers, got %s" % num)
        l.append(num)
    t = None
    for n in l:
        if t is None:
            t = n
        else:
            t = t + n
    return t / len(l)

def averageFromList(l):
    "Extract numbers from a given list and return the average."
    if not isinstance(l, type(["dumb"])):
        raise TypeError(f"Expected lists, got %s" % l)
    ll = list()
    for i in l:
        if isinstance(i, (type(1), type(1.0))):
            ll.append(float(i))
    return sum(ll) / len(ll)



# The number_comparator package.

# It can compare if a number is prime, pair, etc.

from number_comparator.comparisons import *
from number_comparator.operations import *
from number_comparator.data.first_prime import *

__author__ = "Diego Ramirez <dr01191115@gmail.com> @DiddiLeija"
__version__ = "0.0.5"
__license__ = "MIT"



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="number_comparator",
    version="0.0.5.post1",
    author="Diego Ramirez",
    author_email="dr01191115@gmail.com",
    description="A numeric comparator package.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Education",
        "Intended Audience :: Developers",
        'Topic :: Software Development :: Build Tools',
        'Topic :: Scientific/Engineering :: Mathematics',
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    keywords="numbers number comparations operations python",
    python_requires='>=3.7',
)
